// ignore_for_file: avoid_print

import 'package:flutter_test/flutter_test.dart';
import 'package:wiredash/src/core/options/feedback_options.dart';
import 'package:wiredash/src/core/widgets/larry_page_view.dart';
import 'package:wiredash/src/feedback/_feedback.dart';

import 'util/assert_widget.dart';
import 'util/robot.dart';
import 'util/wiredash_tester.dart';

void main() {
  group('Email validation errors', () {
    testWidgets('Submit works with valid email', (tester) async {
      final WiredashTestRobot robot = await goToEmailStep(tester);
      await robot.enterEmail('dash@flutter.io');
      await robot.submitEmailViaButton();
      selectByType(LarryPageView).childByType(Step6Submit).existsOnce();
    });

    testWidgets('Submit works without email', (tester) async {
      final WiredashTestRobot robot = await goToEmailStep(tester);
      await robot.enterEmail('');
      await robot.submitEmailViaButton();
      selectByType(LarryPageView).childByType(Step6Submit).existsOnce();
    });

    testWidgets('Submit via button - Shows error for invalid email',
        (tester) async {
      final WiredashTestRobot robot = await goToEmailStep(tester);
      await robot.enterEmail('invalid');
      await robot.submitEmailViaButton();
      await tester.waitUntil(
        find.text('l10n.feedbackStep4EmailInvalidEmail'),
        findsOneWidget,
      );
    });

    testWidgets('Submit via enter - Shows error for invalid email',
        (tester) async {
      final WiredashTestRobot robot = await goToEmailStep(tester);
      await robot.enterEmail('invalid');
      await robot.submitEmailViaKeyboard();
      await tester.waitUntil(
        find.text('l10n.feedbackStep4EmailInvalidEmail'),
        findsOneWidget,
      );
    });

    testWidgets('Do not ask for email', (tester) async {
      final robot = await WiredashTestRobot.launchApp(tester);

      await robot.openWiredash();
      await robot.enterFeedbackMessage('test message');
      await robot.goToNextStep();
      await robot.skipScreenshot();
      expect(
        robot.services.feedbackModel.feedbackFlowStatus,
        isNot(FeedbackFlowStatus.email),
      );
      expect(
        robot.services.feedbackModel.feedbackFlowStatus,
        FeedbackFlowStatus.submit,
      );
    });
  });
}

Future<WiredashTestRobot> goToEmailStep(WidgetTester tester) async {
  final robot = await WiredashTestRobot.launchApp(
    tester,
    feedbackOptions: const WiredashFeedbackOptions(
      askForUserEmail: true,
    ),
  );

  await robot.openWiredash();
  await robot.enterFeedbackMessage('test message');
  await robot.goToNextStep();
  await robot.skipScreenshot();
  expect(
    robot.services.feedbackModel.feedbackFlowStatus,
    FeedbackFlowStatus.email,
  );
  return robot;
}
